/* 
 E-xmms ver 1.0- A small xmms Epplet 
 Evan Songer <esonger@clarku.edu> 
 This is a GPL Licensed program, you know the deal. 
*/

#include <stdio.h>
#include <gtk/gtk.h> 
#include <gdk/gdkx.h> 
#include <X11/Xlib.h> 
#include <X11/Xatom.h> 
#include <xmms/xmmsctrl.h> 
#include <getopt.h>  
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <epplet.h>

#define MIXER_DEV       "/dev/mixer"
#define XMMS_PROG        "xmms"

Epplet_gadget b_mute, xmms_b, label, label2,  play_b,  pause_b, eject_b, prev_b, next_b, stop_b, close_b, vslider,  hbar; 
 
extern void Epplet_redraw(void); 
 
char *xmmsprog = XMMS_PROG; 
int mp_pid = 0, len = 1, pos = 1, newpos = 1; 
int xmms_session = 0; 
 
static void xmms_cb(void *data); 
static void close_cb(void *data); 
static void play_cb(void); 
static void pause_cb(void);  
static void eject_cb(void); 
static void prev_cb(void);  
static void next_cb(void);  
static void stop_cb(void);   
static void volume_cb(void *data); 
static void slide_cb(void *data); 
static void run_cb(void *data);

int open_mixer(void);
int read_volume(int);
void set_volume(void *);
void initialize_channels(void);

typedef struct _MIX_CHANNEL {
	int id;
	int value;
} MIX_CHANNEL;

int fd_mixer = 0;
int msk_mixerdevs = 0;
MIX_CHANNEL channels[4];
int mute_status = 0;
gboolean running;

static void toggle_mute(void *data)
{
	unsigned char lrvl[4];
	MIX_CHANNEL *dev = (MIX_CHANNEL *)data;

	if (mute_status) {
		lrvl[0] = lrvl[1] = 0;
	} else {
		lrvl[0] = (20 - (unsigned char)dev->value) * 5;
		lrvl[1] = lrvl[0];
	}
	lrvl[2] = lrvl[3] = 0;
	ioctl(fd_mixer, MIXER_WRITE(dev->id), lrvl);
}

int open_mixer(void)
{
	if ((fd_mixer = open(MIXER_DEV, O_RDWR)) < 0) {
		fprintf(stderr, "Mixer device open failed...\n");
		return 1;
	}
	if (ioctl(fd_mixer, SOUND_MIXER_READ_DEVMASK, &msk_mixerdevs) == -1) {
		fprintf(stderr, "Mixer device mask read failed...\n");
		return 1;
	}
	return 1;
}

int read_volume(int channel)
{
	unsigned char lrvl[4];
	if (ioctl(fd_mixer, MIXER_READ(channels[channel].id), lrvl) == -1) {
		return 0;
	}
	channels[channel].value = (100 - (lrvl[0] + lrvl[1])/2) / 5;
	return channels[channel].value;
}

void set_volume(void *data)
{
	unsigned char lrvl[4];
	MIX_CHANNEL *dev = (MIX_CHANNEL *)data;

	if (mute_status) {
		mute_status = 0;
		toggle_mute(&channels[0]);
		Epplet_gadget_data_changed(b_mute);
	}
	lrvl[0] = (20 - (unsigned char)dev->value) * 5;
	lrvl[1] = lrvl[0];
	lrvl[2] = lrvl[3] = 0;
	ioctl(fd_mixer, MIXER_WRITE(dev->id), lrvl);
}

void initialize_channels(void)
{
	channels[0].id = SOUND_MIXER_VOLUME;
	channels[0].value = read_volume(0);
	channels[1].id = SOUND_MIXER_SYNTH;
	channels[1].value = read_volume(1);
	channels[2].id = SOUND_MIXER_PCM;
	channels[2].value = read_volume(2);
	channels[3].id = SOUND_MIXER_LINE;
	channels[3].value = read_volume(3);
	Epplet_timer((void*)initialize_channels, NULL, 0.05, "TIMER2");
}

static void
run_cb(void *data)
{
	running = xmms_remote_is_running(xmms_session);  
	if (running){
		Epplet_gadget_hide(xmms_b);   
                Epplet_gadget_hide(label);   
                Epplet_gadget_hide(close_b);   
                Epplet_gadget_show(play_b);   
                Epplet_gadget_show(pause_b);   
                Epplet_gadget_show(eject_b);   
                Epplet_gadget_show(prev_b);   
                Epplet_gadget_show(next_b);   
                Epplet_gadget_show(stop_b);   
                Epplet_gadget_show(vslider);   
                Epplet_gadget_show(hbar);   
                Epplet_gadget_show(label2); 
	}  else {
		Epplet_gadget_show(xmms_b);   
                Epplet_gadget_show(label);    
                Epplet_gadget_show(close_b);   
                Epplet_gadget_hide(play_b);   
                Epplet_gadget_hide(pause_b);   
                Epplet_gadget_hide(eject_b);   
                Epplet_gadget_hide(prev_b);   
                Epplet_gadget_hide(next_b);   
                Epplet_gadget_hide(stop_b);   
                Epplet_gadget_hide(vslider);   
                Epplet_gadget_hide(hbar);   
                Epplet_gadget_hide(label2); 
	} 
	Epplet_timer(run_cb, NULL, 0.5, "TIMER1"); 
        data = NULL;
}

static void
slide_cb(void *data)
{
	pos = xmms_remote_get_output_time(xmms_session);
	if (pos < 1)
		pos = 1;
	len = xmms_remote_get_playlist_time(xmms_session, xmms_remote_get_playlist_pos(xmms_session));
	if (len < 1 )
		len =1;
	newpos = (pos * 100) / len;
	Epplet_gadget_data_changed(hbar);
	Epplet_timer(slide_cb, NULL, 0.05, "TIMER");
	data = NULL;
}

static void
close_cb(void *data)
{
	Epplet_unremember();
	Esync();
	exit(0);
	data = NULL;
}

static void
xmms_cb(void *data)
{
	mp_pid = Epplet_spawn_command(xmmsprog);
	Epplet_gadget_hide(xmms_b);
	Epplet_gadget_hide(label);
	Epplet_gadget_hide(close_b);
	Epplet_gadget_show(play_b);
	Epplet_gadget_show(pause_b);
	Epplet_gadget_show(eject_b);
	Epplet_gadget_show(prev_b);
	Epplet_gadget_show(next_b);
	Epplet_gadget_show(stop_b);
	Epplet_gadget_show(vslider);
	Epplet_gadget_show(hbar);
	Epplet_gadget_show(label2);
	return;
	data = NULL;
	
}
static void play_cb(void)
{
	xmms_remote_play(xmms_session);
}

static void pause_cb(void) 
{ 
        xmms_remote_pause(xmms_session); 
}
static void eject_cb(void) 
{ 
        xmms_remote_eject(xmms_session); 
} 
static void prev_cb(void)  
{  
        xmms_remote_playlist_prev(xmms_session);  
}
static void next_cb(void)  
{  
        xmms_remote_playlist_next(xmms_session);  
}
static void stop_cb(void)  
{  
        xmms_remote_stop(xmms_session);
	
}


int 
main(int argc, char **argv)
{
	if (!open_mixer())
		return 1;
	initialize_channels();

	Epplet_timer(slide_cb, NULL, 0.05, "TIMER");
	Epplet_timer(run_cb, NULL, 0.5, "TIMER1");
	Epplet_timer((void*)initialize_channels, NULL, 0.05, "TIMER2" );
	Epplet_Init("E-Xmms", "1.0", "E Xmms epplet", 3, 3, argc, argv, 0);
	play_b = Epplet_create_button(NULL, NULL, 9, 22, 0, 0, "PLAY", 0, NULL, (void*)play_cb, NULL);
	pause_b = Epplet_create_button(NULL, NULL, 21, 22, 0, 0, "PAUSE", 0, NULL, (void*)pause_cb, NULL);
	eject_b = Epplet_create_button(NULL, NULL, 33, 22, 0, 0, "ARROW_UP", 0, NULL, (void*)eject_cb, NULL);
	prev_b = Epplet_create_button(NULL, NULL, 9, 34, 0, 0, "PREVIOUS", 0, NULL, (void*)prev_cb, NULL);
	next_b = Epplet_create_button(NULL, NULL, 21, 34, 0, 0, "NEXT", 0, NULL, (void*)next_cb, NULL);
	stop_b = Epplet_create_button(NULL, NULL, 33, 34, 0, 0, "STOP", 0, NULL, (void*)stop_cb, NULL);
	label = Epplet_create_label(5, 20, "XMMS", 3);
	label2 = Epplet_create_label(12, 3, "XMMS", 2);
	xmms_b = Epplet_create_button(NULL, NULL, 2, 2, 0, 0, "SKIP", 0, NULL, xmms_cb, NULL); 
	close_b = Epplet_create_button(NULL, NULL, 33, 2, 0, 0, "CLOSE", 0, NULL, close_cb, NULL);
	vslider = Epplet_create_vslider(2, 2, 44, 0, 20, 1, 5, &channels[2].value, set_volume, &channels[2]);
	hbar = Epplet_create_hbar(9, 14, 35, 8, 0, &newpos);
       
	Epplet_gadget_show(xmms_b);
	Epplet_gadget_show(label);
	Epplet_gadget_show(close_b);

	Epplet_show();
	Epplet_Loop();

	return 0;
}
